;1.4) LoadAndDispatch
; Prototype:
;	int LoadAndDispatch( void *pInterface, DISPATCH *pDispatch );
; Input:
;	pDispatcher:	Pointer to filled DISPATCH structure.
;	pInterface:		Pointer to interface. It's members will be filled on return.
; Output:
;	Returns the number of methods found by the dispatcher.
; Remarks:
;	The return value may not equal iMethods.
;	If a particular method is not found, the corresponding interface structure member remains unchanged.

.code
align DWORD
LoadAndDispatch	proc uses ebx esi edi pDispatch:PTR DISPATCH, pInterface:DWORD
	local iTotal	:SDWORD
	local iDone		:SDWORD

	mov ebx,pDispatch
	mov edi,pInterface
	test ebx,ebx
	jz badarg
	test edi,edi
	jz badarg
	mov esi,[ebx].DISPATCH.pMethods
	mov eax,[ebx].DISPATCH.iMethods
	test esi,esi
	jz badarg
	cmp eax,0
	jle badarg
	mov iTotal,eax
	.if [ebx].DISPATCH.hLib == NULL
		invoke FindDispatchLibrary,ebx
		test eax,eax
		jz quit
		invoke LoadLibrary,eax
		test eax,eax
		jz quit
		mov [ebx].DISPATCH.hLib,eax
	.endif
	mov iDone,0
	.repeat
		mov eax,[esi]
		test eax,eax
		jz notfound
		invoke GetProcAddress,[ebx].DISPATCH.hLib,eax
		test eax,eax
		jz getdefault
found:
		mov [edi],eax
		inc iDone
notfound:
		add esi,4
		add edi,4
		dec iTotal
	.until zero?
	mov eax,iDone

quit:
	ret

getdefault:
	mov eax,[ebx].DISPATCH.hDefLib
	.if eax == NULL
		invoke LoadLibrary,[ebx].DISPATCH.pszDefault
		mov [ebx].DISPATCH.hDefLib,eax
	.endif
	invoke GetProcAddress,eax,dword ptr [esi]
	test eax,eax
	jnz found
	jmp notfound

badarg:
	push ERROR_BAD_ARGUMENTS

seterr:
	call SetLastError
	xor eax,eax
	jmp quit

LoadAndDispatch endp
